--[[===============================================================================================
org.oscillity.MissingKeyActions.xrnx/main.lua
===============================================================================================]]--

--[[

  Everything here is more or less directly adapted from the 
  Renoise GlobalMidiActions.lua file:
  https://github.com/renoise/xrnx/blob/master/GlobalMidiActions.lua

]]


--------------------------------------------------------------------------------
-- Globals (interface to Renoise)
--------------------------------------------------------------------------------

-- max mapping counts
MAX_SECTION_MAPPINGS = 64

---------------------------------------------------------------------------------------------------
-- Helper methods 
---------------------------------------------------------------------------------------------------

local gather_section_positions = function()
  print("gather_section_positions")
  local rns = renoise.song() 
  local positions = {}
  for k,v in ipairs(rns.sequencer.pattern_sequence) do
    if rns.sequencer:sequence_is_start_of_section(k) then
      table.insert(positions,k)
    end
  end
  return positions
end

local set_scheduled_section = function(section_index)
  print("set_scheduled_section",section_index)
  local rns = renoise.song() 
  local positions = gather_section_positions()
  if positions[section_index] then
    rns.transport:set_scheduled_sequence(positions[section_index])
  end
end

local get_section_index_by_seq_pos = function(seq_pos)
  print("get_section_index_by_seq_pos",seq_pos)
  local positions = gather_section_positions()
  if not table.is_empty(positions) then
    for k,v in ipairs(positions) do
      if (v > seq_pos) then
        return k-1
      elseif (v == seq_pos) then
        return k
      end
    end
    return #positions,positions
  end
  
end

-- Seq. Looping:Section 

local loop_section_by_index = function(section_index)
  print("loop_section_by_index",section_index)

  local rns = renoise.song()

  local positions = gather_section_positions()
  if table.is_empty(positions) then
    return
  end
  if not positions[section_index] then
    return
  end

  -- rules: enable loop if partially selected, or unselected
  -- disable loop if section (and _only_ section) is wholly looped

  local section_start = positions[section_index] 
  local section_end = positions[section_index+1] and 
    positions[section_index+1]-1 or #rns.sequencer.pattern_sequence

  local within_range = function(pos,range_start,range_end)
    return pos >= range_start and pos <= range_end
  end

  local enable_loop = false
  local loop_seq_empty = (rns.transport.loop_sequence_range[1] == 0) and 
    (rns.transport.loop_sequence_range[2] == 0)
  if not loop_seq_empty then
    local all_looped,all_empty = false,false
    for k,v in ipairs(rns.sequencer.pattern_sequence) do
      if within_range(k,section_start,section_end) then
        if within_range(k,rns.transport.loop_sequence_start,rns.transport.loop_sequence_end) then
          if not all_looped then
            all_looped = true
          end
          all_empty = false
        else
          if not all_empty then
            all_empty = true
          end
          all_looped = false
        end
      end
    end
    if all_looped then
      enable_loop = false
    elseif all_empty then
      enable_loop = true
    end
  else
    enable_loop = true
  end

  if enable_loop then
    rns.transport.loop_sequence_range = {section_start,section_end}
  else
    rns.transport.loop_sequence_range = {}
  end

end


---------------------------------------------------------------------------------------------------
-- Menu entries & MIDI/Key mappings
---------------------------------------------------------------------------------------------------

for section_index = 1,MAX_SECTION_MAPPINGS do
  renoise.tool():add_keybinding {
    name = string.format(
      "Global:Pattern Sequence:Schedule Section #%02d",
      section_index),
    invoke = function(repeated)
      if not repeated then
        set_scheduled_section(section_index)
      end 
    end
  }
  renoise.tool():add_keybinding {
    name = string.format(
      "Global:Pattern Sequence:Loop Section #%02d",
      section_index),
    invoke = function(repeated)
      if not repeated then
        loop_section_by_index(section_index)
      end 
    end
  }
end

renoise.tool():add_keybinding {
  name = "Global:Pattern Sequence:Loop Current Section",
  invoke = function(repeated)
    local rns = renoise.song() 
    if not repeated then
      local seq_pos = rns.transport.edit_pos.sequence
      local section_index = get_section_index_by_seq_pos(seq_pos)
      loop_section_by_index(section_index)
    end 
  end
}

